﻿using Hotline.Share.Dtos.Push;
using Hotline.Share.Enums.Push;
using MapsterMapper;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using System.Net;
using System.Text.RegularExpressions;
using System.Xml;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Hotline.Push;

/// <summary>
/// 
/// </summary>
public class PushDomainService : IPushDomainService, IScopeDependency
{

    private readonly IMessageRepository _messageRepository;
    private readonly IMapper _mapper;
    private readonly IHttpClientFactory _httpClientFactory;
    private readonly ILogger<PushDomainService> _logger;
    private readonly IConfiguration _config;
    private readonly SmsAccountInfo accountInfo = null;

    /// <summary>
    /// 
    /// </summary>
    /// <param name="messageRepository"></param>
    /// <param name="httpClientFactory"></param>
    /// <param name="logger"></param>
    /// <param name="mapper"></param>
    /// <param name="config"></param>
    public PushDomainService(IMessageRepository messageRepository, IHttpClientFactory httpClientFactory, ILogger<PushDomainService> logger, IMapper mapper, IConfiguration config)
    {
        _messageRepository = messageRepository;
        _httpClientFactory = httpClientFactory;
        _logger = logger;
        _mapper = mapper;
        _config = config;
        accountInfo = _config.GetSection("SmsAccountInfo").Get<SmsAccountInfo>();
    }

    /// <summary>
    /// 业务处理
    /// </summary>
    /// <param name="messageDto"></param>
    /// <param name="cancellation"></param>
    /// <returns></returns>
    public async Task PushAsync(MessageDto messageDto, CancellationToken cancellation)
    {
        if (messageDto == null)
        {
            throw UserFriendlyException.SameMessage("消息不能为空！");
        }

        #region 替换模板内容
        //如果模板为空，参数为空则直接用短信内容
        if (!string.IsNullOrEmpty(messageDto.Template) && messageDto.Params.Count > 0)
        {
            string Template = messageDto.Template;
            //正则查询模板中需要替换的内容
            Regex regex = new(@"\{[a-zA-Z0-9]{1,}\}");
            var matches = regex.Matches(Template);
            if (matches != null && matches.Count != messageDto.Params.Count)
                //参数与需要替换的字符数不匹配
                //throw UserFriendlyException.SameMessage("模板需要参数与实际传递参数个数不匹配！");
                messageDto.Remark = "模板需要参数与实际传递参数个数不匹配！";

            //根据正则查询出来的匹配项替换内容
            for (int i = 0; i < matches.Count; i++)
                Template = Template.Replace(matches[i].ToString(), messageDto.Params[i]);

            messageDto.Content = Template;
        }
        #endregion

        var message = _mapper.Map<Message>(messageDto);

        //调用短信发送业务
        var strResult = await SmsSend(messageDto);

        //处理返回值
        if (true == strResult.Contains("ok,"))  // ok,35797257
        {
            string[] strArray = strResult.Split(',');
            if (2 == strArray.Length)
            {
                // 短信服务中心返回 IFSFWID，用于短信发送状态接收时匹配
                message.SmsWaitSendingId = strArray[1];
                message.SendState = Share.Enums.Push.ESendState.Sending;
            }
        }
        else
        {
            message.Remark = strResult;
        }
        //写入本地数据库
        await _messageRepository.AddAsync(message);
    }

    #region 查询账号短信剩余量
    /// <summary>
    /// 查询账号短信剩余量
    /// </summary>
    /// <returns></returns>
    public async Task<string> GetAccountNum()
    {
        // 账户,密码
        string strAcceptContent = $"{accountInfo.AccountUser},{accountInfo.AccountPwd}";
        // 参数
        Dictionary<string, string> dicParam = new()
        {
            { "AcceptType", "fwAccountNum" },
            { "AcceptContent", strAcceptContent }
        };
        // 将参数转化为HttpContent
        HttpContent content = new FormUrlEncodedContent(dicParam);
        // ok,9
        // error,查询异常
        string strResult = await PostHelper(accountInfo.MessageServerUrl, content);
        var strArray = strResult.Split(",");
        if (2 == strArray.Length)
        {
            strResult = strArray[1];
        }
        else
        {
            strResult = "查询异常";
        }
        return strResult;
    }
    #endregion

    /// <summary>
    /// 短信发送状态回调接口
    /// </summary>
    /// <param name="account">短信回传账号</param>
    /// <param name="pswd">短信回传密码</param>
    /// <param name="msgid">短信中心短信待发送ID，同短信发送接口返回值ID</param>
    /// <param name="sendtime">短信发送时间</param>
    /// <param name="msgcount">发送短信使用的数量</param>
    /// <param name="state">发送状态：0:未发送  1:发送中  2:发送失败  3:发送成功</param>
    /// <param name="errormsg">错误消息</param>
    /// <param name="sfid">短信中心短信已发送ID</param>
    /// <param name="telnumall">手机号码</param>
    /// <param name="sign">短信签名</param>
    /// <returns></returns>
    public async Task<string> ReceiveObtain(string account, string pswd, string msgid, string sendtime, int msgcount, int state, string errormsg, int sfid, string telnumall, string sign)
    {
        string strResult = "error,缺少参数";
        if (account != accountInfo.ReturnAccountUser || pswd != accountInfo.ReturnAccountPwd)
        {
            strResult = "error,参数错误";
            return strResult;
        }
        if (true == string.IsNullOrEmpty(account) || true == string.IsNullOrEmpty(pswd)
            || true == string.IsNullOrEmpty(msgid) || true == string.IsNullOrEmpty(sendtime)
            || msgcount <= 0 || state < 0)
        {
            strResult = "error,参数错误";
            return strResult;
        }

        //修改数据
        var data = await _messageRepository.GetAsync(p => p.SmsWaitSendingId == msgid);
        if (data != null)
        {
            data.SendTime = Convert.ToDateTime(sendtime);
            data.MsgCount = msgcount;
            data.SendState = (ESendState)state;
            data.SmsSendingCompletedId = sfid + "";
            data.Remark = errormsg;
            await _messageRepository.UpdateAsync(data);
            // 成功返回值必须是ok
            strResult = "ok";
        }
        else
            strResult = "error,调用失败";

        return strResult;
    }

    #region 私有方法

    #region 短信发送
    /// <summary>
    /// 短信发送
    /// </summary>
    /// <returns></returns>
    private async Task<string> SmsSend(MessageDto messageDto)
    {
        string strResult = "";
        try
        {
            SendSmsModelDto tSms = new()
            {
                // 业务系统短信ID
                nWaitID = 1,
                // 短信类型1:默认2:营销
                strType = "1",
                // 业务系统工单ID
                nFlowID = 0,
                // 姓名
                strPhoneName = messageDto.Name,
                // 手机号码
                strPhoneNumber = messageDto.TelNumber,
                // 短信内容
                strPhoneConnent = messageDto.Content,
                // 添加时间
                dAddTime = DateTime.Now,
                strPhoneNumberAll = "",
                strRemark = "",
                // 业务系统添加人ID
                nUserID = 0,
                // 业务系统添加人姓名
                strUserName = "",
                // 业务系统添加部门ID
                nBMID = 0,
                // 业务系统添加部门名称
                strBMName = "",
                // 短信系统账号
                strAccountUser = accountInfo.AccountUser,
                // 短信系统密码
                strAccountPwd = accountInfo.AccountPwd
            };
            // 序列化
            string strSendJson = Newtonsoft.Json.JsonConvert.SerializeObject(tSms);
            // 参数加密
            string strKey = await FwEncrypt(strSendJson);
            // 短信发送参数
            Dictionary<string, string> dicParam = new Dictionary<string, string>
            {
                { "AcceptType", "fwSms" },
                { "AcceptContent", strKey }
            };
            // 将参数转化为HttpContent
            HttpContent content = new FormUrlEncodedContent(dicParam);
            strResult = await PostHelper(accountInfo.MessageServerUrl, content);
        }
        catch (Exception ex)
        {
            strResult = ex.Message;
        }
        return strResult;
    }
    #endregion

    #region 参数加密
    /// <summary>
    /// 参数加密
    /// </summary>
    /// <param name="planText"></param>
    /// <returns></returns>
    private async Task<string> FwEncrypt(string planText)
    {
        // 参数
        Dictionary<string, string> dicParam = new Dictionary<string, string>();
        dicParam.Add("AcceptType", "fwEncrypt");
        dicParam.Add("AcceptContent", planText);
        // 将参数转化为HttpContent
        HttpContent content = new FormUrlEncodedContent(dicParam);
        string strResult = await PostHelper(accountInfo.MessageServerUrl, content);
        string strValue = "";
        // ok,XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
        if (true == strResult.Contains("ok,"))
        {
            string[] strArray = strResult.Split(',');
            if (2 == strArray.Length)
            {
                strValue = strArray[1];
            }
        }
        return strValue;
    }
    #endregion

    #region http请求
    /// <summary>
    /// 封装使用HttpClient调用WebService
    /// </summary>
    /// <param name="url">URL地址</param>
    /// <param name="content">参数</param>
    /// <returns></returns>
    private async Task<string> PostHelper(string url, HttpContent content)
    {
        var result = string.Empty;
        try
        {
            using (var client = _httpClientFactory.CreateClient())
            using (var response = await client.PostAsync(url, content))
            {
                if (response.StatusCode == HttpStatusCode.OK)
                {
                    result = await response.Content.ReadAsStringAsync();
                    XmlDocument doc = new XmlDocument();
                    doc.LoadXml(result);
                    result = doc.InnerText;
                }
            }
        }
        catch (Exception ex)
        {
            result = ex.Message;
        }
        return result;
    }
    #endregion

    #endregion
}